
open Format
open Lexing
open Lexer
open Parser
open Ast

let usage = "usage: jc [options] file.java"

let parse_only = ref false
let type_only = ref false

let main = ref ("Main", "main")
let set_main s =
  try 
    let i = String.index s '.' in 
    main := String.sub s 0 i, String.sub s (i+1) (String.length s - i - 1)
  with _ -> raise (Arg.Bad "bad main method")

let spec = 
  ["-parse-only", Arg.Set parse_only, "  stops after parsing";
   "-type-only", Arg.Set type_only, "  stops after typing";
   "-main", Arg.String set_main, "<class.method>  sets main function" ]

let file = 
  let file = ref None in
  let set_file s =  
    if not (Filename.check_suffix s ".java") then 
      raise (Arg.Bad "no .java extension");
    file := Some s
  in
  Arg.parse spec set_file usage;
  match !file with Some f -> f | None -> Arg.usage spec usage; exit 1

let () =
  let c = open_in file in
  let lb = Lexing.from_channel c in
  try
    let p = Parser.compilation_unit Lexer.token lb in
    close_in c;
    if !parse_only then exit 0;
    let p = Typing.compilation_unit p in
    if !type_only then exit 0;
    let m = !main in Compile.compile file p m
  with
    | Lexical_error s -> 
	Location.report file (lexeme_start lb, lexeme_end lb);
	eprintf "lexical error: %s\n@." s;
	exit 1
    | Parsing.Parse_error ->
	Location.report file (lexeme_start lb, lexeme_end lb);
	eprintf "syntax error\n@.";
	exit 1
    | Typing.Error (l, e) ->
	Location.report file l;
	eprintf "%a\n@." Typing.report e;
	exit 1
    | e ->
	eprintf "Anomaly: %s\n@." (Printexc.to_string e);
	exit 2

	
    
  
