/* Une implmentation des listes circulaires doublement chanes. */
class LinkedList {

    private Object head = null;
    private LinkedList next = this;
    private LinkedList prev = this;
    private int size = 0;
    private LinkedList iterator = null;

    /* Construit une nouvelle liste vide. */
    LinkedList() { }

    private LinkedList(LinkedList l) {
	head = l.head;
	next = l.next;
	prev = l.prev;
	iterator = l.iterator;
    }

    private LinkedList(Object elt) {
	head = elt;
    }

    /* Ajoute en position [idx] (0 pour insrer en tte).
       Retourne [true] si [elt] est bien ajoutee ; 
       [false] si [idx] est plus grand que la taille de la liste. */
    boolean add(int idx, Object elt) {
	if (idx < 0 || idx > size)
	    return false;
	add_aux(idx, elt);
	return true;
    }

    private void add_aux(int idx, Object elt) {
	if (idx == 0) {
	    addFirst(elt);
	    return;
	}
	LinkedList cur = get_aux(idx + 1);
	LinkedList l = new LinkedList(elt);
	l.next = cur;
	l.prev = cur.prev;
	cur.prev.next = l;
	cur.prev = l;
	size++;
    }

    /* Ajout en queue. */
    void add(Object elt) {
	add_aux(size, elt);
    }

    /* Ajout en tte. */
    void addFirst(Object elt) {
	if (size > 0) {
	    LinkedList l = new LinkedList(this);
	    next = l;
	    l.prev = this;
	    l.next.prev = l;
	}		
	head = elt;
	size++;
    }

    /* Ajout en queue. */
    void addLast(Object elt) {
	add_aux(size, elt);
    }

    /* Vide la liste. */
    void clear() {
	size = 0;
	head = null;
	next = null;
	prev = null;
	iterator = null;
    }

    /* Retourne [true] si la liste contient [elt]; [false] sinon.
       (tests effectus avec l'galit physique). */
    boolean contains(Object elt) {
	LinkedList cur = this;
	for(int i = 0; i < size; i++) {
	    if (elt == cur.head)
		return true;
	    cur = cur.next;
	}
	return false;
    }

    /* Retourne le idx-ime lment s'il existe; [null] sinon. */
    Object get(int idx) {
	if (idx < 1 || idx > size)
	    return null;
	return get_aux(idx).head;
    }

    /* Retourne le premier lment si la liste est non vide. 
       Plus efficace que [get(1)]. */
    Object getFirst() {
	return head;
    }

    /* Retourne le dernier lment si la liste est non vide. 
       Plus efficace que [get(size)]. */
    Object getLast() {
	if (prev == null)
	    return null;
	return prev.head;
    }

    private LinkedList get_aux(int idx) {
	LinkedList cur = this;
	if (idx <= size / 2)
	    for(int i = 1; i < idx; i++)
		cur = cur.next;
	else
	    for(int i = size; i >= idx; i--)
		cur = cur.prev;
	return cur;
    }

    /* Retourne la 1re position de [elt] dans la liste ou [-1] s'il n'existe
       pas. */
    int indexOf(Object elt) {
	LinkedList cur = this;
	for(int i = 1; i <= size; i++) {
	    if (cur.head == elt)
		return i;
	    cur = cur.next;
	}
	return -1;
    }
    
    /* Retourne la dernire position de [elt] dans la liste ou [-1] s'il
       n'existe pas. */
    int lastIndexOf(Object elt) {
	LinkedList cur = prev;
	for(int i = size; i > 0; i--) {
	    if (cur.head == elt)
		return i;
	    cur = cur.prev;
	}
	return -1;
    }

    /* Supprime le idx-ime lment de la liste s'il existe et le retourne.
       Retourne [null] sinon. 
       L'itrateur est dplac vers l'lment suivant si l'lment supprim
       est celui sur lequel il est. */
    Object remove(int idx) { 
	if (idx <= 0 || idx > size)
	    return null;
	return remove_aux(idx);
    }

    private Object remove_aux(int idx) {
	if (idx == 1) {
	    Object elt = head;
	    return removeFirst();
	}
	LinkedList cur = get_aux(idx);
	LinkedList l = cur.next;
	cur.prev.next = cur.next;
	l.prev = cur.prev;
	size--;
	if (iterator == cur)
	    iterator = l;
	return cur.head;
    }

    /* Supprime [elt] de la liste. Retourne la position de cet lment ou
       [-1] s'il n'existe pas. 
       L'itrateur est dplac vers l'lment suivant si l'lment supprim
       est celui sur lequel il est. */
    int remove(Object elt) {
	LinkedList cur = this;
	for(int i = 1; i <= size; i++) {
	    if (cur.head == elt) {
		remove_aux(i);
		return i;
	    }
	    cur = cur.next;
	}
	return -1;
    }

    /* Supprime le premier lment si la liste est non vide et le retourne.
       Retourne [null] sinon.
       L'itrateur est dplac vers le 2me lment s'il est sur le premier
       lment. */
    Object removeFirst() {
	Object elt = head;
	if (size == 1) {
	    size = 0;
	    head = null;
	    iterator = null;
	} else {
	    head = next.head;
	    LinkedList l = next;
	    next = next.next;
	    next.prev = this;
	    size--;
	    l.clear();
	}
	return elt;
    }

    /* Supprime le dernier lment si la liste est non vide et le retourne.
       Retourne [null] sinon. 
       L'itrateur est dplac vers le 1er lment s'il est sur le dernier
       lment. */
    Object removeLast() {
	return remove(size);
    }

    /* Modifie en place le [idx]-ime lment.
       Retourne l'ancienne valeur (si elle existe); [null] sinon. */
    Object set(int idx, Object elt) {
	if (idx <= 0 || idx > size)
	    return null;
	LinkedList cur = get_aux(idx);
	Object h = cur.head;
	cur.head = elt;
	return h;
    }

    /* Retourne le nombre d'lments. */
    int size() {
	return size;
    }

    /* Dplace l'itrateur vers l'lment suivant et retourne ce dernier. 
       Si l'itrateur est en fin de liste, dplace l'itrateur vers le premier
       lment. 
       Retourne [null] si la liste est vide. */
    Object next() {
	if (iterator == null) {
	    if (size == 0)
		return null;
	    iterator = this;
	} else
	    iterator = iterator.next;
	return iterator.head;
    }

    /* Dplace l'itrateur vers l'lment prcdent et retourne ce dernier.
       Si l'itrateur est en dbut de liste, dplace l'itrateur vers le 
       dernier lment. 
       Retourne [null] si la liste est vide. */
    Object prev() {
	if (iterator == null) {
	    if (size == 0)
		return null;
	    iterator = this.prev;
	} else
	    iterator = iterator.prev;
	return iterator.head;
    }

    /* Affichage d'une liste de Int (debugging purpose) */
    void display() {
	LinkedList cur = this;
	for(int i = 0; i < size; i++) {
	    System.out.println(((Int)cur.head).n + "");
	    cur = cur.next;
	}
    }
}

// Classes de tests

class Int {
    int n;
    Int(int n) { this.n = n; }
}

class Main {
    static String string_of_bool(boolean b) {
	if (b) return "true";
	return "false";
    }

    static void main() {
	LinkedList l = new LinkedList();
	for(int i = 0; i < 5; i++) 
	    l.addFirst(new Int(i));
	boolean b = true;
  	Int quinze = new Int(15);
	b = b && l.add(4, quinze);
	b = b && l.add(2, quinze);
	b = b && l.add(6, new Int(11));
	b = b && ! l.add(9, new Int(2000));
  	l.addLast(new Int(8));
  	l.add(0, new Int(0));
	b = b && l.size() == 10;
	b = b && l.contains(quinze) && ! l.contains(new Int(15));
	b = b && ((Int)l.get(7)).n == 15 && ((Int)l.get(3)).n == 3;
	b = b && l.get(11) == null;
	b = b && ((Int)l.getFirst()).n == 0;
	b = b && ((Int)l.getLast()).n == 8;
	b = b && l.indexOf(quinze) == 4 && l.lastIndexOf(quinze) == 7;
	b = b && ((Int)l.remove(4)).n == 15 && ((Int)l.remove(1)).n == 0;
	b = b && ((Int)l.remove(8)).n == 8;
	b = b && l.remove(8) == null;
	b = b && ((Int)l.removeFirst()).n == 4 && ((Int)l.removeLast()).n == 0;
	b = b && ((Int)l.set(2, quinze)).n == 2 && ((Int)l.get(2)).n == 15;
	b = b && ((Int)l.set(4, new Int(7))).n == 15 && ((Int)l.get(4)).n == 7;
	b = b && l.set(6, quinze) == null;
    	b = b && ((Int)l.next()).n == 3;
    	b = b && ((Int)l.next()).n == 15;
    	b = b && ((Int)l.next()).n == 1;
    	b = b && ((Int)l.next()).n == 7;
    	b = b && ((Int)l.next()).n == 11;
    	b = b && ((Int)l.next()).n == 3;
    	b = b && ((Int)l.prev()).n == 11;
    	b = b && ((Int)l.prev()).n == 7;
    	b = b && ((Int)l.prev()).n == 1;
    	b = b && ((Int)l.prev()).n == 15;
    	b = b && ((Int)l.prev()).n == 3;
    	b = b && ((Int)l.prev()).n == 11;

	if (b)
	    System.out.println("ok");
    }
}
